﻿using C1.WPF.FlexGrid;
using FilterRow;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace FlexGrid_CollectionChanged
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
            ObservableCollection<Employee> _list = new ObservableCollection<Employee>();

        public MainWindow()
        {
            InitializeComponent();
            _grid.ItemsSource = EmployeeList;
          //  _grid.ColumnHeaders.Rows.Add(new Row() { IsReadOnly=false});
            _grid.CellFactory = new CustomFactory();
            _grid.Rows.CollectionChanged += Rows_CollectionChanged;
        }
        private void Rows_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine($"Action: {e.Action}");  
        }
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            //           _grid.Rows.Add(new Row() {  DataItem="123"});

            PopulateList();
        }

        private void PopulateList()
        {
            EmployeeList.Clear();
            _list.Add(new Employee() { FirstName = "John", LastName = "Wayne", Age = 35, JoiningDate = new DateTime(2007, 12, 1), IncrementInSalary = .16 });
            _list.Add(new Employee() { FirstName = "Maximus", LastName = "Decimus", Age = 33, JoiningDate = new DateTime(2011, 05, 15), IncrementInSalary = .65 });
            _list.Add(new Employee() { FirstName = "Alexander", LastName = "Conklin", Age = 42, JoiningDate = new DateTime(2004, 06, 22), IncrementInSalary = .63 });
            _list.Add(new Employee() { FirstName = "William", LastName = "Bourne", Age = 66, JoiningDate = new DateTime(2015, 04, 15), IncrementInSalary = .64 });
            _list.Add(new Employee() { FirstName = "Jacob", LastName = "Lee", Age = 26, JoiningDate = new DateTime(2017, 08, 11), IncrementInSalary = .18 });
            _list.Add(new Employee() { FirstName = "Mike", LastName = "Bourne", Age = 56, JoiningDate = new DateTime(1999, 1, 21), IncrementInSalary = 1.41 });
            _list.Add(new Employee() { FirstName = "Fuller", LastName = "Brown", Age = 36, JoiningDate = new DateTime(2012, 08, 07), IncrementInSalary = .34 });
            _list.Add(new Employee() { FirstName = "Miller", LastName = "Wayne", Age = 42, JoiningDate = new DateTime(2014, 07, 19), IncrementInSalary = .95 });
            _list.Add(new Employee() { FirstName = "Sam", LastName = "Conklin", Age = 22, JoiningDate = new DateTime(2018, 06, 15), IncrementInSalary = .13 });
            _list.Add(new Employee() { FirstName = "Mark", LastName = "Bourne", Age = 24, JoiningDate = new DateTime(2016, 06, 15), IncrementInSalary = .02 });
        }
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            EmployeeList.Clear();
        }
        public ObservableCollection<Employee> EmployeeList => _list;

        public class Employee
        {
            public string FirstName { get; set; }
            public string LastName { get; set; }
            public int Age { get; set; }
            public DateTime JoiningDate { get; set; }
            public double IncrementInSalary { get; set; }
        }
    }
    public class CustomFactory: CellFactory
    {
        public bool UseRegEx { get; set; }
        static Thickness _editorPadding = new Thickness(0, 0, 2, 0);

        //public override void CreateColumnHeaderContent(C1FlexGrid grid, Border bdr, CellRange rng)
        //{
        //    base.CreateColumnHeaderContent(grid, bdr, rng);
        //    if(rng.Row2>0)
        //    {
        //        var child=bdr.Child;
        //        if((child as TextBlock) !=null)
        //        {
        //            var txt = (child as TextBlock).Text;
        //            TextBox txtBx = new TextBox();
        //            txtBx.Text = txt;
        //            bdr.Padding = new Thickness(0);
        //            bdr.Margin = new Thickness(0);
        //            bdr.Child = txtBx;
        //        }
        //    }
        //}
        public override void CreateColumnHeaderContent(C1FlexGrid grid, System.Windows.Controls.Border bdr, CellRange range)
        {
            // create the filter row if it isn't there
            var rows = grid.ColumnHeaders.Rows;
            if (rows.Count == 0 || !(rows[rows.Count - 1] is GridFilterRow))
            {
                rows.Add(new GridFilterRow());
            }

            // add filter editors to filter row
            var fr = rows[range.Row] as GridFilterRow;
            if (fr != null)
            {
                bdr.Child = new FilterEditor(fr, grid.Columns[range.Column]);
                bdr.Padding = _editorPadding;
            }
            else
            {
                base.CreateColumnHeaderContent(grid, bdr, range);
            }
        }
        public override void CreateTopLeftContent(C1FlexGrid grid, System.Windows.Controls.Border bdr, CellRange range)
        {
            var rows = grid.ColumnHeaders.Rows;
            var fr = rows[range.Row] as GridFilterRow;
            if (fr != null && range.Column == grid.TopLeftCells.Columns.Count - 1)
            {
                bdr.Child = new ClearFilterButton(fr);
            }
            else
            {
                base.CreateTopLeftContent(grid, bdr, range);
            }
        }
        public override bool IsSortSymbolRow(C1FlexGrid grid, CellRange range)
        {
            return range.TopRow == grid.ColumnHeaders.Rows.Count - 2;
        }
    }
    public class GridFilterRow : Row
    {
        // ** fields
        Dictionary<Column, string> _filterArgs = new Dictionary<Column, string>();
        Dictionary<Column, string> _rxPattern = new Dictionary<Column, string>();
        bool _useRegex;

        // ** object model
        public string GetFilterArgument(Column col)
        {
            string text;
            return _filterArgs.TryGetValue(col, out text)
                ? text
                : string.Empty;
        }
        public void SetFilterArgument(Column col, string arg)
        {
            if (arg != GetFilterArgument(col))
            {
                // save argument
                _filterArgs[col] = arg;

                // save regex pattern to support wildcards
                arg = arg != null ? arg.Replace("%", ".*") : arg;
                _rxPattern[col] = string.Format("^{0}$", arg);

                // apply the filter
                ApplyFilter();
            }
        }

        // ** implementation
        void ApplyFilter()
        {
            // check whether to use wildcards when matching
            var cf = Grid.CellFactory as CustomFactory;
            _useRegex = cf != null && cf.UseRegEx;

            // apply filter to grid's ICollectionView
            var view = Grid.ItemsSource as ICollectionView;
            if (view == null || !view.CanFilter)
            {
                // apply the filter in unbound mode
                foreach (var row in Grid.Rows)
                {
                    row.Visible = FilterPredicate(row);
                }

                // done
                return;
            }

            // can't apply filters while editing or adding items
            var edt = view as IEditableCollectionView;
            if (edt != null)
            {
                if (edt.IsEditingItem)
                    edt.CommitEdit();

                if (edt.IsAddingNew)
                    edt.CommitNew();
            }

            // apply the filter in bound mode
            try
            {
                view.Filter = FilterPredicate;
            }
            catch (Exception x)
            {
                MessageBox.Show("Failed to apply the filter:\r\n", x.Message);
            }
        }
        bool FilterPredicate(object item)
        {
            // get item type to retrieve properties
            var itemType = item.GetType();

            // in unbound mode, the item is the row itself
            bool unbound = item is Row;

            // check filter values applied to each column
            foreach (var kv in _filterArgs)
            {
                // no filter condition on this column? continue
                if (string.IsNullOrEmpty(kv.Value))
                {
                    continue;
                }

                // get cell value
                var stringValue = string.Empty;
                var row = item as Row;
                if (row != null)
                {
                    // handle unbound mode
                    var value = row[kv.Key];
                    stringValue = value == null ? string.Empty : value.ToString();
                }
                else
                {
                    // get property name
                    var propName = kv.Key.BoundPropertyName;
                    if (string.IsNullOrEmpty(propName))
                    {
                        continue;
                    }

                    // get property info
                    var prop = itemType.GetProperty(propName);
                    if (prop == null)
                    {
                        continue;
                    }

                    // get property value
                    var value = prop.GetValue(item, null);
                    stringValue = value == null ? string.Empty : value.ToString();
                }

                if (_useRegex)
                {
                    // test using regular expressions (support wildcards)
                    if (!Regex.IsMatch(stringValue, _rxPattern[kv.Key], RegexOptions.IgnoreCase))
                    {
                        return false;
                    }
                }
                else
                {
                    // text using simple 'contains' 
                    if (stringValue.IndexOf(kv.Value, StringComparison.OrdinalIgnoreCase) < 0)
                    {
                        return false;
                    }
                }
            }

            // item passed all filter conditions
            return true;
        }
    }

}
